from django.contrib.auth import get_user_model
from rest_framework.authtoken.models import Token
from rest_framework.test import APITestCase, APIRequestFactory


"""
# =============================================================================================
# BASE API TEST CASE ==========================================================================
# =============================================================================================
"""


class BaseAPITestCase(APITestCase):
    username = "testuser1"
    email = "test@test.com"
    username = "api-test-user"
    password = "password"

    def setUp(self):
        super(APITestCase, self).setUp()

        # store standard api factory
        self.factory = APIRequestFactory()

        # create user
        self.user = get_user_model().objects.create_user(
            username=self.username, email=self.email, password=self.password
        )
        self.user_token = Token.objects.create(user=self.user)



# Generated by Django 4.0.4 on 2022-05-30 22:14

import uuid

import django.db.models.deletion
from django.db import migrations, models

import django_dans_notifications.models.email


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="NotificationBasic",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("datetime_created", models.DateTimeField(auto_now_add=True)),
                ("datetime_modified", models.DateTimeField(auto_now=True)),
                ("datetime_sent", models.DateTimeField(blank=True, null=True)),
                ("sent_successfully", models.BooleanField(default=False)),
                (
                    "sender",
                    models.CharField(
                        help_text="This should be the sending users email.",
                        max_length=300,
                    ),
                ),
                (
                    "recipients",
                    models.CharField(
                        help_text="Comma separated list of email recipients.",
                        max_length=900,
                    ),
                ),
                ("read", models.BooleanField(default=False)),
                ("message", models.CharField(max_length=600)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="NotificationEmailTemplate",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("datetime_created", models.DateTimeField(auto_now_add=True)),
                ("datetime_modified", models.DateTimeField(auto_now=True)),
                ("path", models.CharField(max_length=300)),
                ("nickname", models.CharField(max_length=300)),
            ],
            options={
                "abstract": False,
            },
            managers=[
                (
                    "objects",
                    django_dans_notifications.models.email.NotificationEmailTemplateManager(),
                ),
            ],
        ),
        migrations.CreateModel(
            name="NotificationPush",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("datetime_created", models.DateTimeField(auto_now_add=True)),
                ("datetime_modified", models.DateTimeField(auto_now=True)),
                ("datetime_sent", models.DateTimeField(blank=True, null=True)),
                ("sent_successfully", models.BooleanField(default=False)),
                (
                    "sender",
                    models.CharField(
                        help_text="This should be the sending users email.",
                        max_length=300,
                    ),
                ),
                (
                    "recipients",
                    models.CharField(
                        help_text="Comma separated list of email recipients.",
                        max_length=900,
                    ),
                ),
                ("message", models.CharField(max_length=300)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="NotificationEmail",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("datetime_created", models.DateTimeField(auto_now_add=True)),
                ("datetime_modified", models.DateTimeField(auto_now=True)),
                ("datetime_sent", models.DateTimeField(blank=True, null=True)),
                ("sent_successfully", models.BooleanField(default=False)),
                (
                    "sender",
                    models.CharField(
                        help_text="This should be the sending users email.",
                        max_length=300,
                    ),
                ),
                (
                    "recipients",
                    models.CharField(
                        help_text="Comma separated list of email recipients.",
                        max_length=900,
                    ),
                ),
                ("subject", models.CharField(max_length=300)),
                ("context", models.JSONField(blank=True, null=True)),
                (
                    "template",
                    models.ForeignKey(
                        default=django_dans_notifications.models.email.get_default_template,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="template",
                        to="django_dans_notifications.notificationemailtemplate",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                (
                    "objects",
                    django_dans_notifications.models.email.NotificationEmailManager(),
                ),
            ],
        ),
    ]



from ..base import BaseModelTestCase
from ....models.email import NotificationEmailTemplate

"""
# ========================================================================= #
# TEST EMAIL NOTIFICATION TEMPLATE ======================================== #
# ========================================================================= #
"""


class TestEmailNotificationTemplate(BaseModelTestCase):
    model = NotificationEmailTemplate

    def setUp(self):
        super(TestEmailNotificationTemplate, self).setUp()

    # =================================================================== #
    # BASIC TESTS ======================================================= #
    # =================================================================== #

    def test_str(self):
        self.email_template_nickname = "template1"
        self.email_template = self.model.objects.create(
            nickname=self.email_template_nickname,
        )

        self.assertNotEqual(str(self.email_template), None)

    def test_with_path_exists(self):
        self.email_template_nickname = "template1"
        self.email_template_path = "django-dans-emails/template.html"
        self.email_template = self.model.objects.create(
            nickname=self.email_template_nickname, path=self.email_template_path
        )

        self.assertEqual(self.email_template.path, self.email_template_path)

    def test_with_path_exists_no_extension(self):
        self.email_template_nickname = "template1"
        self.email_template_path = "django-dans-emails/template"
        self.email_template = self.model.objects.create(
            nickname=self.email_template_nickname, path=self.email_template_path
        )

        self.assertEqual(self.email_template.path, self.email_template_path)

    def test_with_path_does_not_exist(self):
        self.email_template_nickname = "template1"
        self.email_template_path = "INVALID"
        self.email_template = self.model.objects.create(
            nickname=self.email_template_nickname, path=self.email_template_path
        )

        self.assertEqual(self.email_template.path, self.email_template_path)



from rest_framework import serializers


"""
# ===================================================================================
# BASE SERIALIZER ===================================================================
# ===================================================================================
"""


class BaseSerializer(serializers.ModelSerializer):
    """
    BaseSerializer

    features
    - dynamic fields: takes an additional `fields` argument that controls which fields should be displayed.
    """

    def __init__(self, *args, **kwargs):
        # Don't pass the 'fields' arg up to the superclass
        fields = kwargs.pop("fields", None)

        # Instantiate the superclass normally
        super(BaseSerializer, self).__init__(*args, **kwargs)

        if fields is not None:
            # Drop any fields that are not specified in the `fields` argument.
            allowed = set(fields)
            existing = set(self.fields)
            for field_name in existing - allowed:
                self.fields.pop(field_name)



from django.core.exceptions import ValidationError
from django.utils import timezone
from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated

from ..helpers import api_response_error, api_response_success
from ..models.push import NotificationPush
from ..serializers.push import NotificationPushSerializer
from django.db.models import Q

"""
============================================================================================ #
PUSH ======================================================================================= #
============================================================================================ #
"""


# =============================== #
#
# NOTIFICATION PUSH VIEW SET
#
class NotificationPushViewSet(viewsets.GenericViewSet):
    queryset = NotificationPush.objects.all()
    serializer_class = NotificationPushSerializer
    permission_classes = (IsAuthenticated,)

    def list(self, request, *args, **kwargs):
        """
        Endpoint to list out Push Notifications
        """
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(
            self.filter_queryset(
                self.queryset.filter(
                    Q(recipients__contains=request.user.email)
                    | Q(recipients__contains=request.user.id)
                )
            ),
            many=True,
            context={"request": request},
        )
        page = self.paginate_queryset(serializer.data)
        return self.get_paginated_response(page)

    def retrieve(self, request, *args, **kwargs):
        """
        Endpoint to retrieve specific Push Notification

        :param uuid pk: UUID of NotificationPush to retrieve
        """
        pk = self.kwargs.get("pk")
        try:
            notification_push = NotificationPush.objects.get(pk=pk)
            if not notification_push.recipients_contains(request.user):
                raise NotificationPush.DoesNotExist
        except (NotificationPush.DoesNotExist, ValidationError):
            return api_response_error("Notification not found.")

        # serializer and return
        serializer = self.get_serializer_class()(
            notification_push, context={"request": request}
        )
        return api_response_success(serializer.data)

    def create(self, request, *args, **kwargs):
        """
        Endpoint to create Push Notification
        """
        request_data_copy = request.data.copy()
        request_data_copy["sender"] = request.user.email
        request_data_copy["datetime_sent"] = timezone.now()
        request_data_copy["sent_successfully"] = True

        try:
            serializer_class = self.get_serializer_class()
            serializer = serializer_class(request_data_copy)
            NotificationPush.objects.create(**serializer.data)
        except (ValidationError, TypeError) as e:
            return api_response_error(
                "Error creating notification. Please try again later."
            )

        try:
            return api_response_success(serializer.data)
        except (AttributeError,) as e:
            return api_response_error(e)
